<?php
/**
 * This class is a product view helper for the MAGEconnect Extension
 * 
 * @package TYPO3
 * @subpackage Fluid
 * @version
 */

require_once(t3lib_extMgm::extPath('mageconnect').'lib/class.tx_mageconnect_product.php');
require_once(t3lib_extMgm::extPath('mageconnect').'lib/class.tx_mageconnect_helper.php');
 
class Tx_Mageconnect_ViewHelpers_ProductsByFilterViewHelper extends Tx_Fluid_Core_ViewHelper_AbstractViewHelper
{
	const EXT_NAME = 'mageconnect';

	/**
	 * @var Tx_Extbase_Object_ObjectManagerInterface
	 */
	protected $objectManager;
	
	/**
	 * @var tx_mageconnect_helper
	 */
	protected $_helper;
	
	/**
	 * @var array
	 */
	protected $_cache;
	
	/**
	 * @var tx_mageconnect_product
	 */
	protected $_productClass;

	public function __construct()
	{
		$this->_helper 			= t3lib_div::makeInstance('tx_mageconnect_helper'); 
		$this->_productClass	= t3lib_div::makeInstance('tx_mageconnect_product');
	}

	/**
	 * Injects the Object Manager
	 *
	 * @param Tx_Extbase_Object_ObjectManagerInterface $objectManager
	 * @return void
	 */
	public function injectObjectManager(Tx_Extbase_Object_ObjectManagerInterface $objectManager) {
		$this->objectManager = $objectManager;
	}
	
	/**
	 * Fetch product attribute
	 * 
	 * @param string $tags Magento Product Tags
	 * @param string $categories Magento Categories
	 * @param string $store Magento Store View
	 * @param int $pid Page ID where the products are stored
	 * @param bool $refreshAll Refresh all products
	 * @param bool $enableFilterCache Enable the filter cache
	 * @validate $tags StringValidator
	 * @validate $categories StringValidator
	 * @validate $storeView StringValidator
	 * @validate $pid IntegerValidator
	 * @validate $refreshAll NotEmptyValidator
	 * @validate $enableFilterCache NotEmptyValidator
	 * @return string Content
	 */
	public function render($tags, $categories, $store = "", $pid, $refreshAll = false, $enableFilterCache = true)
	{
		// Check that the divider of the data is comma separation
		$dividers = array(';','/','.',"\\",":","-");
		foreach ($dividers as $divider) {
			$tags = str_replace($divider, ',', $tags);
			$categories = str_replace($divider, ',', $categories);
		}
		
		$tagArr = array();
		if ($tags != "")
		{
			$tagArr 		= explode(',', $tags);
		}
		
		$categoryArr = array();
		if ($categories != "")
		{
			$categoryArr	= explode(',', $categories);
		}

		// Trim
		$tagArr = array_map("trim", $tagArr);
		$categoryArr = array_map("trim", $categoryArr);
		
		$productIds = $this->_productClass->getProductIdsByFilter($tagArr, $categoryArr, $store, $enableFilterCache);
		
		$productData 	= array();
		
		// If we found product ids we need to check if they exist
		if (is_array($productIds) && count($productIds) >= 1) {
			
			$needToImport = array();
			if ((bool)$refreshAll === false) {
				// Get all existing products
				$existing = $this->_productClass->getExistingProducts($store, $pid);
			
				// Get all products that need to be imported
				foreach ($productIds as $_productId) {
					if (!in_array($_productId, $existing)) {
						$needToImport[] = $_productId;
					}				
				}
			} else {
				$needToImport = $productIds;
			}
			
			if (count($needToImport) >= 1) {
				// Import products that need to be imported
				$result = $this->_productClass->importProducts($needToImport, $store, $pid);			
			}
		
			// Fetch products from database	
			$products = $this->_productClass->getProductsFromDb($productIds, $store, $pid);
		
			if ($products) {
				
				
				foreach ($productIds as $_id) {
				
					foreach ($products as $_product) {
				
						if ($_product['product_id'] == $_id) {
							$attributes = array();
							$attributes = unserialize( $_product['attributes'] );
							unset($_product['attributes']);
							$attributes = array_merge($attributes, $_product);
		
							$data = array();
							foreach ($attributes as $_attr=>$_val) {
								if (!is_array($_val)) {
									$data[$_attr] = $_val;
									
								}
							}
							$productData[] = $data;
						}
					}
				}
				$this->templateVariableContainer->add('products', $productData);
			}
		}
		return $productData;
	}
	
	
	
}
