<?php
class tx_mageconnect_userauth 
{
	/**
	 * Log out current user!
	 * Removes the current session record, sets the internal ->user array to a blank string; Thereby the current user (if any) is effectively logged out!
	 *
	 * @return	void
	 */
	function main(&$cObj) 
	{
		if($this->_getParam('logintype') == 'logout') {
				
			$redir = $this->_getModuleConfig('redirect_on_logout');	
			$GLOBALS['TSFE']->fe_user->removeSessionData();	
				
			if (!$this->_getParam('redir')) {
				
				$param = '';
					
				switch ($redir) {
					
					case 'TYPO3':
						$param = '?redir=t3';
						break;
					case 'Magento':
						$param = '?redir=mage';
						break;
					default:
						$param = '?source=t3';
				}
				
				// If no single typo3 logout, redirect using global settings
				if ($this->_getParam('single') != 'true') {
					$this->_redirectMagentoLogout($param);
				}
				
			} else {
				
				// when Magento returns to this procedure
				// and we need to go back to the configured system
				
				if ($redir != 'TYPO3' && $redir != 'Magento') {
					// If logout has to be system of logout,
					// we need to go to base url to clean parameters in url
					$source = $this->_getParam('source');
					
					if ($source == 't3') {
						$this->_redirectTYPO3BaseUrl();
						exit();
					} else {
						$this->_redirectMagentoBaseUrl();
						exit();
					}
					
				} else {
					
					switch ($this->_getParam('redir')) {
						case 'mage':
							$this->_redirectMagentoBaseUrl();
							exit();
							break;
						case 'config':
							$this->_redirectByConfig();
							break;
					}
				}
			}
		}
	}
	
	/**
	 * _redirectByConfig
	 * Checks config and does an final redirect
	 * with the setting that was set in config
	 * 
	 * @return void
	 */
	function _redirectByConfig()
	{
		$redir = $this->_getModuleConfig('redirect_on_logout');
					
		switch ($redir) {
					
			case 'TYPO3':
				$this->_redirectTYPO3BaseUrl();
				exit();
				break;
			case 'Magento':
				$this->_redirectMagentoBaseUrl();
				exit();
				break;
		}
	}
	
	
	
	/**
	 * _getModuleConfig
	 * Gets Module Config for MAGEconnect
	 * 
	 * @param string $key
	 * @return string|bool
	 */
	function _getModuleConfig($key = '')
	{
		$modConfigArray = unserialize( $GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['mageconnect'] );
		if ($modConfigArray[$key])
		{
			return $modConfigArray[$key];
		}
		return false;
	}
	
	/**
	 * _redirectMagentoLogout
	 * Redirects to magento logout url
	 * 
	 */
	function _redirectMagentoLogout($params = '')
	{
		$url = $this->_getModuleConfig('magento_url') . 'customer/account/logout/' . $params;
		header('location: ' . $url);	
		return;
	}

	/**
	 * _redirectMagentoBaseUrl
	 * Redirects to magento base url
	 * 
	 * @return void
	 */
	function _redirectMagentoBaseUrl()
	{
		$url = $this->_getModuleConfig('magento_url');
		header('location: ' . $url);	
		return;
	}

	/**
	 * _redirectTYPO3BaseUrl
	 * Redirects to TYPO3 base url
	 * 
	 * @return void
	 */
	function _redirectTYPO3BaseUrl()
	{
		$requestUrl = $_SERVER["HTTP_REFERER"];
		header("location: " . $requestUrl);
		return;
	}
	
	/**
	 * _getParam
	 * Gets an parameter 
	 * 
	 * @param string Parameter
	 * @return string|bool
	 * 
	 */	
	function _getParam($parameter)
	{
		if (t3lib_div::_GP($parameter) && $parameter != '')	
			return t3lib_div::_GP($parameter);
			
		return false;	
	}
	
}
