<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2011 Bastian Zagar (magedeveloper.de) <kontakt@magedeveloper.de>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 */
require_once(PATH_tslib.'class.tslib_pibase.php');

require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_product.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_helper.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_cache.php');

/**
 * Plugin 'MageConnect Product Display' for the 'mageconnect' extension.
 *
 * @author	Bastian Zagar (magedeveloper.de) <kontakt@magedeveloper.de>
 * @package	TYPO3
 * @subpackage tx_mageconnect
 */
class tx_mageconnect_pi2 extends tslib_pibase {
	
	var $prefixId      = 'tx_mageconnect_pi2';		// Same as class name
	var $scriptRelPath = 'pi2/class.tx_mageconnect_pi2.php';	// Path to this script relative to the extension dir.
	var $extKey        = 'mageconnect';	// The extension key.
	var $pi_checkCHash = true;
	
	var $_helper;
	var $_products;
	var $_pid;
	var $conf;
	
	var $_debugData = array();	
	
	/**
	 * __construct
	 * Constructor
	 */
	function __construct()
	{
		parent::__construct();
		$this->_helper 		= t3lib_div::makeInstance('tx_mageconnect_helper'); 
	}
	
	/**
	 * _getFlexformConfig
	 * Get the flexform config by typename and fieldname
	 * 
	 * @param string $type
	 * @param string $field
	 * @return string
	 */
	function _getFlexformConfig($field, $type)
	{
		return $this->pi_getFFValue($this->cObj->data['pi_flexform'], $field, $type); 
	}	
	
	/**
	 * getProductsStoragePid
	 * Gets the storage pid for the product data either from the config setting
	 * or flexform configuration
	 * 
	 * @return int Storage PID
	 */
	function getProductsStoragePid()
	{
		$configPid = $this->_getFlexformConfig('productsPid', 'general_setting');
		
		if (!$configPid || $configPid == '') {
			return $this->_getConfig('productsStoragePid');
		}
		return $configPid;
	}

	/**
	 * getProductStoragePid
	 * Gets the storage pid for the category data either from the config setting
	 * or flexform configuration
	 * 
	 * @return int Storage PID
	 */
	function getCategoriesStoragePid()
	{
		$configPid = $this->_getFlexformConfig('categoriesPid', 'general_setting');
		
		if (!$configPid || $configPid == '') {
			return $this->_getConfig('categoriesStoragePid');
		}
		return $configPid;
	}
	
	/**
	 * getProductDisplayType
	 * Determine the product display type that
	 * was made in the flexform config of the
	 * plugin
	 * 
	 * @return string
	 */
	function getProductDisplayType()
	{
		return $this->_getFlexformConfig('display_type', 'product_setting');
	}
	
	/**
	 * getStoreViewCode
	 * Get the store view code from the flexform or config setting
	 * 
	 * @return string
	 */
	function getStoreViewCode()
	{
		$configStore = $this->_getFlexformconfig('store_view_code', 'product_setting');
		
		if (!$configStore) {
			return $this->_getConfig('storeViewCode');
		}
		return $configStore;
	}
	
	/**
	 * _getConfig
	 * Return config data from a given
	 * key
	 * 
	 * @param string $key
	 * @return string
	 */
	function _getConfig($key)
	{
		return $this->conf[$key];
	}
	
	/**
	 * reloadProducts
	 * Get the reload configuration setting
	 * 
	 * @return bool
	 */
	function reloadProducts()
	{
		$config = $this->_getFlexformConfig('always_load_from_soap', 'general_setting');
		
		if ($config === '1') {
			return true;
		} 
		return false;
	}
	
	/**
	 * isManualProductSelection
	 * Determines if flexform setting was manual product selection
	 * 
	 * @return bool
	 */
	function isManualProductSelection()
	{
		$productsFrom = $this->_getFlexformConfig('products_from', 'product_setting');
		
		if ($productsFrom == 'products_from_select') {
			return true;
		}
		return false;
	}
	
	/**
	 * getFlexformSelectedCategoryId
	 * Gets the category id that was selected in the flexform
	 * 
	 * @return int
	 */
	function getFlexformSelectedCategoryId()
	{
		return $this->_getFlexformConfig('category_single', 'product_setting');
	}
	
	/**
	 * getFlexformSelectedProducts
	 * Determine the products ids that have been
	 * set in the backend
	 * 
	 * @return array
	 */
	function getFlexformSelectedProducts()
	{
		$products = $this->_getFlexformConfig('product_single', 'product_setting');
		$products = t3lib_div::trimExplode(',',$products,1);
		
		return $products;
	}	
	
	/**
	 * getSelectedProducts
	 * Gets the selected products and imports
	 * the ones, that are needed or forced by reload config
	 * 
	 * @return array
	 */
	function getSelectedProducts()
	{
		$selectedProducts = array();
		
		$piVars = t3lib_div::_GP('tx_mageconnect');
		
		// Store View
		$store = $this->getStoreViewCode();

		// Check if parameter is dynamic detail view -> entity_id
		if ($piVars && array_key_exists('entity_id', $piVars) && $this->getProductDisplayType() === 'dynamic_details') {
				
			$productId = (int)$piVars['entity_id'];	
				
			if ($this->_products->isAllowedDynamicDetailView($productId)) {
				$selectedProducts = array($productId);
			} else {
				return false;
			}
			
		} else {
			
			if ($this->isManualProductSelection()) {
				$selectedProducts = $this->getFlexformSelectedProducts();
			} else {
				
				// Selected Category
				$categoryId 		= $this->getFlexformSelectedCategoryId();
				// Categories Storage Pid
				$categoryStoragePid = $this->getCategoriesStoragePid();
				
				$selectedProducts = $this->_products->getCategoryProducts($categoryId, $categoryStoragePid, $store);
			}
		}
		
		$pid = $this->getProductsStoragePid();
		
		// If we don't need to reload all products, we want only new ones!
		if (!$this->reloadProducts()) {
			
			// Check if Product Data already exists in database
			$existingProducts = $this->_products->getExistingProducts($store, $pid);
			
			
			$productsToFetch = array_diff($selectedProducts, $existingProducts);
			
			$this->_debugData['existing_products'] = $existingProducts;
			$this->_debugData['products_to_fetch'] = $productsToFetch;
			
			if (count($productsToFetch) > 0) {
				// If products don't exist in database, we need to fetch them by soap
				
				$result = $this->_products->importProducts($productsToFetch, $store, $pid);			
			}
			
		} else {
			// Import all products
			$result = $this->_products->importProducts($selectedProducts, $store, $pid);	
		}
		
		$this->_debugData['selected_products'] = $selectedProducts;
		
		return $selectedProducts;
	}
	
	
	/******************************************
	 * DISPLAY FUNCTIONS
	 *****************************************/
	
	/**
	 * displayProductList
	 * Displays a product list
	 * 
	 * @return string
	 */
	function displayProductList()
	{
		// Store View
		$store = $this->getStoreViewCode();
		
		$selectedProducts = $this->getSelectedProducts();
		$products = $this->_products->getProductsFromDb($selectedProducts, $store, $this->getProductsStoragePid());
		
		// Load Template File
		$templateHtml 		= $this->cObj->fileResource( $this->_getConfig('templateProductList') );
		$productListHtml 	= $this->cObj->getSubpart($templateHtml, '###PRODUCT_LIST###');
		$productHtml		= $this->cObj->getSubpart($productListHtml, '###PRODUCT_ITEM###');
		
		$subPartContent = ''; $item = 0; $items = count($products); $even = true; $line = '';
		foreach ($selectedProducts as $_id) {
			foreach ($products as $_product) {
				
				if ($_id === $_product['product_id']) {
					$markers = $this->_products->getProductMarkers($_product);

					// Even/Odd CSS Class Determination
					if ($even === true) {
						$line = 'even';
						$even = false;
					} else {
						$line = 'odd';
						$even = true;
					}
					
					// Class Determination First/Last
					if ($item == 0) {
						$markers['###EVENODD###'] = $line . ' ' . 'first'; 
					} else if ($item == $items-1) {
						$markers['###EVENODD###'] = $line . ' ' . 'last';  
					} else {
						$markers['###EVENODD###'] = $line; 
					}
					
					// Check if the product has an image
					$imageHtml = '<p>'.$this->pi_getLL('template_label_no_image_available').'</p>';
					if ($markers['###DETAIL_IMAGE###'] != 'no_selection') {
						$imageHtml = $this->cObj->getSubpart($productHtml, '###PRODUCT_IMAGE###');
					}
					
					$p = $this->cObj->substituteSubpart($productHtml, '###PRODUCT_IMAGE###', $imageHtml);
					$subPartContent .= $this->cObj->substituteMarkerArray($p, $markers);
					$item++;
				}
			}
		}
		return $this->cObj->substituteSubpart($productListHtml, '###PRODUCT_ITEM###', $subPartContent);
	}
	
	/**
	 * displayProductGrid
	 * Displays a product grid
	 * 
	 * @return string
	 */
	function displayProductGrid()
	{
		// Store View
		$store = $this->getStoreViewCode();
		
		$selectedProducts = $this->getSelectedProducts();
		$products = $this->_products->getProductsFromDb($selectedProducts, $store, $this->getProductsStoragePid());
		
		// Load Template File
		$templateHtml 		= $this->cObj->fileResource( $this->_getConfig('templateProductGrid') );
		$productListHtml 	= $this->cObj->getSubpart($templateHtml, '###PRODUCT_LIST###');
		$productHtml		= $this->cObj->getSubpart($productListHtml, '###PRODUCT_ITEM###');
		
		$subPartContent = '';
		foreach ($selectedProducts as $_id) {
			foreach ($products as $_product) {
				
				if ($_id === $_product['product_id']) {
					$markers = $this->_products->getProductMarkers($_product);
					
					// Check if the product has an image
					$imageHtml = '<p>'.$this->pi_getLL('template_label_no_image_available').'</p>';
					if ($markers['###DETAIL_IMAGE###'] != 'no_selection') {
						$imageHtml = $this->cObj->getSubpart($productHtml, '###PRODUCT_IMAGE###');
					}
					
					$p = $this->cObj->substituteSubpart($productHtml, '###PRODUCT_IMAGE###', $imageHtml);
					$subPartContent .= $this->cObj->substituteMarkerArray($p, $markers);
				}
			}
		}
		return $this->cObj->substituteSubpart($productListHtml, '###PRODUCT_ITEM###', $subPartContent);
	}
	
	/**
	 * displayProductSingle
	 * Displays a small box with
	 * product information
	 * 
	 * @return string
	 */
	function displayProductSingle()
	{
		// Store View
		$store = $this->getStoreViewCode();
		
		$selectedProducts = $this->getSelectedProducts();
		$products = $this->_products->getProductsFromDb($selectedProducts, $store, $this->getProductsStoragePid());
		
		// Load Template File
		$templateHtml 		= $this->cObj->fileResource( $this->_getConfig('templateProductSingle') );
		$productListHtml 	= $this->cObj->getSubpart($templateHtml, '###PRODUCT_LIST###');
		$productHtml		= $this->cObj->getSubpart($productListHtml, '###PRODUCT_ITEM###');
		
		$subPartContent = '';
		foreach ($selectedProducts as $_id) {
			foreach ($products as $_product) {
				
				if ($_id === $_product['product_id']) {
					$markers = $this->_products->getProductMarkers($_product);
					
					// Check if the product has an image
					$imageHtml = '<p>'.$this->pi_getLL('template_label_no_image_available').'</p>';
					if ($markers['###DETAIL_IMAGE###'] != 'no_selection') {
						$imageHtml = $this->cObj->getSubpart($productHtml, '###PRODUCT_IMAGE###');
					}
					
					$p = $this->cObj->substituteSubpart($productHtml, '###PRODUCT_IMAGE###', $imageHtml);
					$subPartContent .= $this->cObj->substituteMarkerArray($p, $markers);
				}
			}
		}
		return $this->cObj->substituteSubpart($productListHtml, '###PRODUCT_ITEM###', $subPartContent);
	}
	
	/**
	 * displayProductDetail
	 * Display product detail information
	 * 
	 * @return string
	 */
	function displayProductDetail()
	{
		// Store View
		$store = $this->getStoreViewCode();
		
		$selectedProducts = $this->getSelectedProducts();
		
		if ($selectedProducts === false) {
			return $this->pi_getLL('template_label_product_not_allowed_for_dynamic_details');
		}
		
		$products = $this->_products->getProductsFromDb($selectedProducts, $store, $this->getProductsStoragePid());
		
		// Load Template File
		$templateHtml 		= $this->cObj->fileResource( $this->_getConfig('templateProductDetail') );
		$detailHtml  		= $this->cObj->getSubpart($templateHtml, '###PRODUCT_DETAIL###');
		$productHtml 		= $this->cObj->getSubpart($templateHtml, '###PRODUCT_ITEM###');
		$attributeHtml		= $this->cObj->getSubpart($templateHtml, '###PRODUCT_ATTRIBUTES###');
		$attributePart		= $this->cObj->getSubpart($templateHtml, '###ATTRIBUTES###');
		
		$subPartContent = '';
		foreach ($selectedProducts as $_k=>$_id) {
			
			foreach ($products as $_product) {
				
				if (trim($_id) === $_product['product_id']) {
					
					$markers = $this->_products->getProductMarkers($_product);
					
					$attributes = json_decode($_product['attributes'], true);
					$attributes = $attributes['additional_attributes'];
					
					// Create Attribute Table
					$attributeSubPart = ''; $attributeMarkers = array(); $attributeInfo = '';
					if (count($attributes) > 0) {
						foreach ($attributes as $_attribute) {
							$attributeMarkers['###ATTRIBUTE_NAME###'] = $_attribute['label'];
							$attributeMarkers['###ATTRIBUTE_VALUE###'] = $_attribute['value'];
							$attributeSubPart .= $this->cObj->substituteMarkerArray($attributePart, $attributeMarkers);
						}
						// Fill final attribute table with single rows
						$attributeInfo = $this->cObj->substituteSubpart($attributeHtml, '###ATTRIBUTES###', $attributeSubPart);
					}

					
					// Check if the product has an image
					$imageHtml = '<p>'.$this->pi_getLL('template_label_no_image_available').'</p>';
					if ($markers['###DETAIL_IMAGE###'] != 'no_selection') {
						$imageHtml = $this->cObj->getSubpart($productHtml, '###PRODUCT_IMAGE###');
					}
					// Insert Image to Template
					$p = $this->cObj->substituteSubpart($productHtml, '###PRODUCT_IMAGE###', $imageHtml);
					
					// Insert Finished Attribute Table to Template
					$p = $this->cObj->substituteSubpart($p, '###PRODUCT_ATTRIBUTES###', $attributeInfo);
					
					$subPartContent .= $this->cObj->substituteMarkerArray($p, $markers);
				}
			}
		}
		return $this->cObj->substituteSubpart($detailHtml, '###PRODUCT_ITEM###', $subPartContent);
	}

	/**
	 * displayProductMarkers
	 * Display product available template markers
	 * 
	 * @return string
	 */
	function displayProductMarkers()
	{
		// Store View
		$store = $this->getStoreViewCode();
		
		$selectedProducts = $this->getSelectedProducts();
		$products = $this->_products->getProductsFromDb($selectedProducts, $store, $this->getProductsStoragePid());
		
		// Load Template File
		$templateHtml 		= $this->cObj->fileResource( $this->_getConfig('templateProductMarkers') );
		$markerListHtml		= $this->cObj->getSubpart($templateHtml, '###MARKER_LIST###');
		$productListHtml	= $this->cObj->getSubpart($markerListHtml, '###PRODUCT_LIST###');
		$markerDetailHtml	= $this->cObj->getSubpart($productListHtml, '###MARKER_DETAIL###');
		
		$subPartContent = '';
		foreach ($selectedProducts as $_id) {
			foreach ($products as $_product) {	
				
				if ($_id === $_product['product_id']) {
					
					$markers = $this->_products->getProductMarkers($_product);
					
					$markerSubContent = '';
					foreach ($markers as $_marker=>$_value) {
						$markerArr = array();
						$markerArr['###MARKER###'] = $_marker;
						$markerArr['###VALUE###'] = htmlentities( $_value );
						$markerSubContent .= $this->cObj->substituteMarkerArray($markerDetailHtml, $markerArr);
					}
					
					$subPartContent .= $this->cObj->substituteSubpart($productListHtml, '###MARKER_DETAIL###', $markerSubContent);
				}	
			}
		}
		return $subPartContent;
	}
	/*****************************************/	
	
	
	
		
	/**
	 * The main method of the PlugIn
	 *
	 * @param	string		$content: The PlugIn content
	 * @param	array		$conf: The PlugIn configuration
	 * @return	The content that is displayed on the website
	 */
	function main($content, $conf) 
	{
		
		$this->pi_initPIflexForm();    // Initialize the FlexForms array 
		
		$this->conf = $conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		$this->pi_USER_INT_obj = 0;
		
		// Register Objects
		tx_mageconnect_registry::set($conf, 'conf');
		tx_mageconnect_registry::set($this->cObj, 'cObj');
		tx_mageconnect_registry::set($this, 'controller');
		
		// Initialize Products Class
		$this->_products = t3lib_div::makeInstance('tx_mageconnect_product');
		$this->_products->initRegistry();

		if ($this->getProductsStoragePid()) {

			// Determine display type		
			$displayType = $this->getProductDisplayType();
			$this->_debugData['display_type'] = $displayType;
			
			switch ($displayType) {
				
					// Products list
					case 'display_list':
						$content .= $this->displayProductList();
						break;
					// Products grid
					case 'display_grid':
						$content .= $this->displayProductGrid();
						break;	
					// Single product
					case 'display_single':
						$content .= $this->displayProductSingle();
						break;	
					// Product details
					case 'display_detail':
						$content .= $this->displayProductDetail();
						break;	
					// Product markers
					case 'display_markers':
						$content .= $this->displayProductMarkers();
						break;	
					case 'dynamic_details':
						$content .= $this->displayProductDetail();
						break;
					default:
						$content .= '';
						break;	
					
			}
		} else {
			$content = $this->pi_getLL('template_label_no_sysfolder_set');
		}
		
		// Show Debug Information
		if ($this->_helper->isInDebugMode() == true) {
			t3lib_utility_Debug::debug($this->_debugData);
		}
		
		return $this->pi_wrapInBaseClass($content);
	}
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/pi2/class.tx_mageconnect_pi2.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/pi2/class.tx_mageconnect_pi2.php']);
}


?>